@@ -1,41 +1,28 @@
 /*
- * Copyright (c) 1998-2001 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1998-2003 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
- *
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
- *
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * 
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * 
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
- *
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
+ * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
-/*
- * Copyright (c) 2000-2001 Apple Computer, Inc.  All rights reserved.
- *
- * IOATABlockStorageDevice.cpp
- *
- * This subclass implements a relay to a protocol and device-specific
- * provider.
- *
- *
- * HISTORY
- *
- *		09/28/2000	CJS		Started IOATABlockStorageDevice
- *							(ported IOATAHDDriveNub)
- *
- */
-
 #include <IOKit/IOLib.h>
 #include <IOKit/IOKitKeys.h>
 #include <IOKit/storage/IOStorageDeviceCharacteristics.h>
@@ -89,7 +76,7 @@ IOATABlockStorageDevice::attach ( IOService * provider )
 		
 	}
 	
-	reserved = ( ExpansionData * ) IOMalloc ( sizeof ( ExpansionData ) );
+	reserved = IONew ( ExpansionData, 1 );
 	if ( reserved == NULL )
 	{
 		
@@ -100,7 +87,7 @@ IOATABlockStorageDevice::attach ( IOService * provider )
 	
 	bzero ( reserved, sizeof ( ExpansionData ) );
 	
-	dictionary = OSDictionary::withCapacity ( 1 );
+	dictionary = OSDictionary::withCapacity ( 2 );
 	if ( dictionary != NULL )
 	{
 		
@@ -229,7 +216,7 @@ IOATABlockStorageDevice::detach ( IOService * provider )
 			
 		}
 		
-		IOFree ( reserved, sizeof ( ExpansionData ) );
+		IODelete ( reserved, ExpansionData, 1 );
 		reserved = NULL;
 		
 	}
@@ -358,6 +345,12 @@ IOATABlockStorageDevice::setProperties ( OSObject * properties )
 	
 	STATUS_LOG ( ( "IOATABlockStorageDevice::setProperties called\n" ) );
 	
+	status = super::setProperties ( properties );
+	if ( status == kIOReturnSuccess )
+	{
+		return status;
+	}
+	
 	number = ( OSNumber * ) fProvider->getProperty ( kIOATASupportedFeaturesKey );
 	features = number->unsigned32BitValue ( );
 	
@@ -718,6 +711,42 @@ IOATABlockStorageDevice::reportWriteProtection ( bool * isWriteProtected )
 }
 
 
+//---------------------------------------------------------------------------
+// 
+
+IOReturn
+IOATABlockStorageDevice::getWriteCacheState ( bool * enabled )
+{
+	
+	// Block incoming I/O if we have been terminated
+	if ( isInactive ( ) != false )
+	{
+		return kIOReturnNotAttached;
+	}
+	
+	return fProvider->getWriteCacheState ( enabled );
+	
+}
+
+
+//---------------------------------------------------------------------------
+// 
+
+IOReturn
+IOATABlockStorageDevice::setWriteCacheState ( bool enabled )
+{
+	
+	// Block incoming I/O if we have been terminated
+	if ( isInactive ( ) != false )
+	{
+		return kIOReturnNotAttached;
+	}
+	
+	return fProvider->setWriteCacheState ( enabled );
+	
+}
+
+
 //---------------------------------------------------------------------------
 // 
 
