@@ -1,40 +1,28 @@
 /*
- * Copyright (c) 1998-2001 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1998-2003 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
- *
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
- *
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * 
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * 
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
- *
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
+ * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
-/*
- * Copyright (c) 2000-2001 Apple Computer, Inc.  All rights reserved.
- *
- * IOATABlockStorageDevice.h
- *
- * This subclass implements a relay to a protocol and device-specific
- * provider.
- *
- *
- * HISTORY
- *
- *		09/28/2000	CJS		Started IOATABlockStorageDevice
- *							(ported IOATAHDDrive)
- *
- */
 
 #ifndef _IO_ATA_BLOCKSTORAGE_DEVICE_H_
 #define _IO_ATA_BLOCKSTORAGE_DEVICE_H_
@@ -63,7 +51,7 @@ protected:
 	};
 	ExpansionData * reserved;
 	
-	#define fClients	reserved->fClients
+	#define fClients			reserved->fClients
 	
 	virtual bool		attach ( IOService * provider );
 	virtual void		detach ( IOService * provider );
@@ -129,6 +117,9 @@ public:
     
     virtual IOReturn	reportWriteProtection ( bool * isWriteProtected );
 	
+	virtual IOReturn	getWriteCacheState ( bool * enabled );
+	virtual IOReturn	setWriteCacheState ( bool enabled );
+	
 	/* Added with 10.1.4 */
 	OSMetaClassDeclareReservedUsed ( IOATABlockStorageDevice, 1 )
 	
