@@ -1,34 +1,30 @@
 /*
- * Copyright (c) 1998-2001 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1998-2003 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
- *
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
- *
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * 
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * 
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
- *
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
+ * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
-/*
- * Copyright (c) 2000-2001 Apple Computer, Inc.  All rights reserved.
- *
- * HISTORY
- *
- *		09/28/2000	CJS		Started IOATABlockStorageDevice
- *							(ported IOATAHDDrive)
- *
- */
+#ifndef _IOKIT_IO_ATA_STORAGE_DEFINES_H_
+#define _IOKIT_IO_ATA_STORAGE_DEFINES_H_
 
 
 /*
@@ -216,6 +212,18 @@ enum
 	kATADataIsValidMask						= 0xC000
 };
 
+/* Bits for features published in Word 85 of device identify data */
+enum
+{
+	kATAWriteCacheEnabledBit				= 5
+};
+
+/* Masks for features published in Word 85 of device identify data */
+enum
+{
+	kATAWriteCacheEnabledMask				= (1 << kATAWriteCacheEnabledBit)
+};
+
 
 /* ATA supported features */
 enum
@@ -255,4 +263,7 @@ enum
 	kATAEnableUltraDMAModeMask 		= 0x40,
 	kATAEnableMultiWordDMAModeMask	= 0x20,
 	kATAEnablePIOModeMask			= 0x08,
-};
\ No newline at end of file
+};
+
+
+#endif	/* _IOKIT_IO_ATA_STORAGE_DEFINES_H_ */
\ No newline at end of file
